@@ -7,6 +7,7 @@ from typing import Union
 from urllib.robotparser import RobotFileParser
 
 from .connection import CloudflareScraper
+from .connection import CookieDict
 from .connection import Response
 from .connection import get
 from .connection import get_robots
@@ -39,7 +40,7 @@ class FAAPI:
     This class provides the methods to access and parse Fur Affinity pages and retrieve objects.
     """
 
-    def __init__(self, cookies: Union[list[dict[str, str]], CookieJar]):
+    def __init__(self, cookies: Union[list[CookieDict], CookieJar]):
         """
         :param cookies: The cookies for the session.
         """
@@ -64,7 +65,7 @@ class FAAPI:
         """
         return float(self.robots.crawl_delay(self.user_agent) or 1)
 
-    def load_cookies(self, cookies: Union[list[dict[str, str]], CookieJar]):
+    def load_cookies(self, cookies: Union[list[CookieDict], CookieJar]):
         """
         Load new cookies and create a new session.
 
@@ -163,7 +164,7 @@ class FAAPI:
         """
         page_parsed: BeautifulSoup = self.get_parsed("/")
         submissions: list[SubmissionPartial] = [SubmissionPartial(f) for f in parse_submission_figures(page_parsed)]
-        return sorted({s.id: s for s in submissions}.values(), reverse=True)
+        return sorted({s for s in submissions}, reverse=True)
 
     def submission(self, submission_id: int, get_file: bool = False, *, chunk_size: int = None
                    ) -> tuple[Submission, Optional[bytes]]:
@@ -209,13 +210,13 @@ class FAAPI:
         return User(self.get_parsed(join_url("user", username_url(user))))
 
     # noinspection DuplicatedCode
-    def gallery(self, user: str, page: int = 1) -> tuple[list[SubmissionPartial], int]:
+    def gallery(self, user: str, page: int = 1) -> tuple[list[SubmissionPartial], Optional[int]]:
         """
         Fetch a user's gallery page.
 
         :param user: The name of the user (_ characters are allowed).
         :param page: The page to fetch.
-        :return: A list of SubmissionPartial objects and the next page (0 if it is the last).
+        :return: A list of SubmissionPartial objects and the next page (None if it is the last).
         """
         page_parsed: BeautifulSoup = self.get_parsed(join_url("gallery", username_url(user), int(page)))
         info_parsed: dict[str, Any] = parse_user_submissions(page_parsed)
@@ -227,16 +228,16 @@ class FAAPI:
         ]
         for s in (submissions := list(map(SubmissionPartial, info_parsed["figures"]))):
             s.author = author
-        return submissions, (page + 1) * (not info_parsed["last_page"])
+        return submissions, (page + 1) if not info_parsed["last_page"] else None
 
     # noinspection DuplicatedCode
-    def scraps(self, user: str, page: int = 1) -> tuple[list[SubmissionPartial], int]:
+    def scraps(self, user: str, page: int = 1) -> tuple[list[SubmissionPartial], Optional[int]]:
         """
         Fetch a user's scraps page.
 
         :param user: The name of the user (_ characters are allowed).
         :param page: The page to fetch.
-        :return: A list of SubmissionPartial objects and the next page (0 if it is the last).
+        :return: A list of SubmissionPartial objects and the next page (None if it is the last).
         """
         page_parsed: BeautifulSoup = self.get_parsed(join_url("scraps", username_url(user), int(page)))
         info_parsed: dict[str, Any] = parse_user_submissions(page_parsed)
@@ -248,28 +249,28 @@ class FAAPI:
         ]
         for s in (submissions := list(map(SubmissionPartial, info_parsed["figures"]))):
             s.author = author
-        return submissions, (page + 1) * (not info_parsed["last_page"])
+        return submissions, (page + 1) if not info_parsed["last_page"] else None
 
-    def favorites(self, user: str, page: str = "") -> tuple[list[SubmissionPartial], str]:
+    def favorites(self, user: str, page: str = "") -> tuple[list[SubmissionPartial], Optional[str]]:
         """
         Fetch a user's favorites page.
 
         :param user: The name of the user (_ characters are allowed).
         :param page: The page to fetch.
-        :return: A list of SubmissionPartial objects and the next page ("" if it is the last).
+        :return: A list of SubmissionPartial objects and the next page (None if it is the last).
         """
         page_parsed: BeautifulSoup = self.get_parsed(join_url("favorites", username_url(user), page.strip()))
         info_parsed: dict[str, Any] = parse_user_favorites(page_parsed)
         submissions: list[SubmissionPartial] = list(map(SubmissionPartial, info_parsed["figures"]))
-        return submissions, info_parsed["next_page"]
+        return submissions, info_parsed["next_page"] or None
 
-    def journals(self, user: str, page: int = 1) -> tuple[list[JournalPartial], int]:
+    def journals(self, user: str, page: int = 1) -> tuple[list[JournalPartial], Optional[int]]:
         """
         Fetch a user's journals page.
 
         :param user: The name of the user (_ characters are allowed).
         :param page: The page to fetch.
-        :return: A list of Journal objects and the next page (0 if it is the last).
+        :return: A list of Journal objects and the next page (None if it is the last).
         """
         page_parsed: BeautifulSoup = self.get_parsed(join_url("journals", username_url(user), int(page)))
         info_parsed: dict[str, Any] = parse_user_journals(page_parsed)
@@ -281,15 +282,15 @@ class FAAPI:
         ]
         for j in (journals := list(map(JournalPartial, info_parsed["sections"]))):
             j.author = author
-        return journals, (page + 1) * (not info_parsed["last_page"])
+        return journals, (page + 1) if not info_parsed["last_page"] else None
 
-    def watchlist_to(self, user: str, page: int = 1) -> tuple[list[UserPartial], int]:
+    def watchlist_to(self, user: str, page: int = 1) -> tuple[list[UserPartial], Optional[int]]:
         """
         Fetch a page from the list of users watching the user.
 
         :param user: The name of the user (_ characters are allowed).
         :param page: The page to fetch.
-        :return: A list of UserPartial objects and the next page (0 if it is the last).
+        :return: A list of UserPartial objects and the next page (None if it is the last).
         """
         users: list[UserPartial] = []
         us, np = parse_watchlist(
@@ -299,14 +300,14 @@ class FAAPI:
             _user.name = u
             _user.status = s
             users.append(_user)
-        return users, (page + 1) if np else 0
+        return users, np if np and np != page else None
 
-    def watchlist_by(self, user: str, page: int = 1) -> tuple[list[UserPartial], int]:
+    def watchlist_by(self, user: str, page: int = 1) -> tuple[list[UserPartial], Optional[int]]:
         """
         Fetch a page from the list of users watched by the user.
         :param user: The name of the user (_ characters are allowed).
         :param page: The page to fetch.
-        :return: A list of UserPartial objects and the next page (0 if it is the last).
+        :return: A list of UserPartial objects and the next page (None if it is the last).
         """
         users: list[UserPartial] = []
         us, np = parse_watchlist(
@@ -316,4 +317,4 @@ class FAAPI:
             _user.name = u
             _user.status = s
             users.append(_user)
-        return users, (page + 1) if np else 0
+        return users, np if np and np != page else None
