@@ -1,32 +1,42 @@
 from datetime import datetime
-from re import compile as re_compile
-from re import IGNORECASE
-from re import match
+from re import MULTILINE
+from re import Match
 from re import Pattern
+from re import compile as re_compile
+from re import match
 from re import search
 from re import sub
 from typing import Any
 from typing import Optional
+from typing import Union
 
+from bbcode import Parser as BBCodeParser  # type:ignore
 from bs4 import BeautifulSoup
 from bs4.element import NavigableString
 from bs4.element import Tag
 from dateutil.parser import parse as parse_date
+from htmlmin import minify  # type:ignore
 
 from .connection import root
-from .exceptions import _raise_exception
 from .exceptions import DisabledAccount
+from .exceptions import NoTitle
 from .exceptions import NonePage
 from .exceptions import NotFound
 from .exceptions import NoticeMessage
-from .exceptions import NoTitle
 from .exceptions import ParsingError
 from .exceptions import ServerError
+from .exceptions import _raise_exception
 
-mentions_regexp: Pattern = re_compile(r"^(?:(?:https?://)?(?:www\.)?furaffinity.net)?/user/([^/#]+).*$")
-watchlist_next_regexp: Pattern = re_compile(r"/watchlist/(by|to)/[^/]+/\d+")
+relative_url: Pattern = re_compile(r"^(?:https?://(?:www\.)?furaffinity\.net)?(.*)")
+mentions_regexp: Pattern = re_compile(r"^(?:(?:https?://)?(?:www\.)?furaffinity\.net)?/user/([^/#]+).*$")
+watchlist_next_regexp: Pattern = re_compile(r"/watchlist/(?:by|to)/[^/]+/(\d+)")
 not_found_messages: tuple[str, ...] = ("not in our database", "cannot be found", "could not be found", "user not found")
 deactivated_messages: tuple[str, ...] = ("deactivated", "pending deletion")
+smilie_icons: tuple[str, ...] = (
+    "crying", "derp", "dunno", "embarrassed", "evil", "gift", "huh", "lmao", "love", "nerd", "note", "oooh", "pleased",
+    "rollingeyes", "sad", "sarcastic", "serious", "sleepy", "smile", "teeth", "tongue", "veryhappy", "wink", "yelling",
+    "zipped", "angel", "badhairday", "cd", "coffee", "cool", "whatever"
+)
 
 
 def get_attr(tag: Tag, attr: str) -> str:
@@ -69,10 +79,10 @@ def inner_html(tag: Tag) -> str:
 
 
 def clean_html(html: str) -> str:
-    return sub(r" {2,}", " ", sub(r"[\r\n]", "", html)).strip()
+    return sub(r" *(<br/?>) *", r"\1", minify(html, remove_comments=True, reduce_boolean_attributes=True)).strip()
 
 
-def html_to_bbcode(html: str, *, special_characters: bool = False) -> str:
+def html_to_bbcode(html: str) -> str:
     body: Optional[Tag] = parse_page(f"<html><body>{html}</body></html>").select_one("html > body")
     if not body:
         return ""
@@ -99,25 +109,28 @@ def html_to_bbcode(html: str, *, special_characters: bool = False) -> str:
 
     for span in body.select("span.bbcode[style*=color]"):
         if m := match(r".*color: ?([^ ;]+).*", span.attrs["style"]):
-            span.replaceWith(f"[color={m[1]}]{html_to_bbcode(inner_html(span))}[/color]")
+            span.replaceWith(f"[color={m[1]}]", *span.children, "[/color]")
         else:
-            span.replaceWith(html_to_bbcode(inner_html(span)))
+            span.replaceWith(*span.children)
 
     for nav_link in body.select("span.parsed_nav_links"):
         a_tags = nav_link.select("a")
-        a_prev_tag = next(filter(lambda t: t.text.strip().upper() == "<<<\xA0PREV", a_tags), None)
-        a_frst_tag = next(filter(lambda t: t.text.strip().upper() == "FIRST", a_tags), None)
-        a_last_tag = next(filter(lambda t: t.text.strip().upper() == "NEXT\xA0>>>", a_tags), None)
+        a_prev_tag: Optional[Tag] = next((a for a in a_tags if "prev" in a.text.lower()), None)
+        a_frst_tag: Optional[Tag] = next((a for a in a_tags if "first" in a.text.lower()), None)
+        a_next_tag: Optional[Tag] = next((a for a in a_tags if "next" in a.text.lower()), None)
         a_prev = a_prev_tag.attrs.get("href", "").strip("/").split("/")[-1] if a_prev_tag else ""
         a_frst = a_frst_tag.attrs.get("href", "").strip("/").split("/")[-1] if a_frst_tag else ""
-        a_last = a_last_tag.attrs.get("href", "").strip("/").split("/")[-1] if a_last_tag else ""
-        nav_link.replaceWith(f"[{a_prev or '-'},{a_frst or '-'},{a_last or '-'}]")
+        a_next = a_next_tag.attrs.get("href", "").strip("/").split("/")[-1] if a_next_tag else ""
+        nav_link.replaceWith(f"[{a_prev or '-'},{a_frst or '-'},{a_next or '-'}]")
 
-    for a in body.select("a.auto_link_shortened"):
-        a.replaceWith(f"[url={a.attrs.get('href', '')}]{a.attrs.get('href', '')}[/url]")
+    for a in body.select("a.auto_link_shortened:not(.named_url), a.auto_link:not(.named_url)"):
+        a.replaceWith(a.attrs.get('href', ''))
 
     for a in body.select("a"):
-        a.replaceWith(f"[url={a.attrs.get('href', '')}]{html_to_bbcode(inner_html(a))}[/url]")
+        href_match: Optional[Match] = relative_url.match(a.attrs.get('href', ''))
+        a.replaceWith(f"[url={href_match[1] if href_match else a.attrs.get('href', '')}]",
+                      *a.children,
+                      "[/url]")
 
     for yt in body.select("iframe[src*='youtube.com/embed']"):
         yt.replaceWith(f"[yt]https://youtube.com/embed/{yt.attrs.get('src', '').strip('/').split('/')}[/yt]")
@@ -129,14 +142,14 @@ def html_to_bbcode(html: str, *, special_characters: bool = False) -> str:
             quote_name_tag.replaceWith(quote_author)
             continue
         quote_name_tag.decompose()
-        quote_tag.replaceWith(f"[quote{('=' + quote_author) if quote_author else ''}]"
-                              f"{html_to_bbcode(inner_html(quote_tag))}"
+        quote_tag.replaceWith(f"[quote{('=' + quote_author) if quote_author else ''}]",
+                              *quote_tag.children,
                               "[/quote]")
 
     for quote_tag in body.select("span.bbcode.bbcode_quote"):
-        quote_tag.replaceWith(f"[quote]{html_to_bbcode(inner_html(quote_tag))}[/quote]")
+        quote_tag.replaceWith("[quote]", *quote_tag.children, "[/quote]")
 
-    for [selector, bbcode] in (
+    for [selector, bbcode_tag] in (
             ("i", "i"),
             ("b", "b"),
             ("strong", "b"),
@@ -156,34 +169,162 @@ def html_to_bbcode(html: str, *, special_characters: bool = False) -> str:
             ("h6", "h6"),
     ):
         for tag in body.select(selector):
-            tag.replaceWith(f"[{bbcode}]{html_to_bbcode(inner_html(tag))}[/{bbcode}]")
+            tag.replaceWith(f"[{bbcode_tag}]", *tag.children, f"[/{bbcode_tag}]")
 
-    for tag in body.select(":not(br)"):
+    for br in body.select("br"):
+        br.replaceWith("\n")
+
+    for p in body.select("p"):
+        p.replaceWith(*p.children)
+
+    for tag in body.select("*"):
         if not (div_class := tag.attrs.get("class", None)):
-            tag.replaceWith(f"[tag.{tag.name}]{html_to_bbcode(inner_html(tag))}[/tag.{tag.name}]")
+            tag.replaceWith(f"[tag={tag.name}]", *tag.children, "[/tag.{tag.name}]")
         else:
-            tag.replaceWith(f"[tag.{tag.name}.{' '.join(div_class) if isinstance(div_class, list) else div_class}]"
-                            f"{html_to_bbcode(inner_html(tag))}"
-                            f"[/tag.{tag.name}]")
+            tag.replaceWith(f"[tag={tag.name}.{' '.join(div_class) if isinstance(div_class, list) else div_class}]",
+                            *tag.children,
+                            "[/tag]")
 
-    html = body.decode_contents()
+    bbcode: str = body.decode_contents()
 
-    if special_characters:
-        for char, substitution in (
-                (r"<br/?>", "\n"),
-                ("©", "(c)"),
-                ("™", "(tm)"),
-                ("®", "(r)"),
-                ("&copy;", "(c)"),
-                ("&reg;", "(tm)"),
-                ("&trade;", "(r)"),
-                ("&lt;", "<"),
-                ("&gt;", ">"),
-                ("&amp;", "&"),
-        ):
-            html = sub(char, substitution, html, flags=IGNORECASE)
+    bbcode = sub(" *$", "", bbcode, flags=MULTILINE)
+    bbcode = sub("^ *", "", bbcode, flags=MULTILINE)
 
-    return html.strip()
+    for char, substitution in (
+            ("©", "(c)"),
+            ("™", "(tm)"),
+            ("®", "(r)"),
+            ("&copy;", "(c)"),
+            ("&reg;", "(tm)"),
+            ("&trade;", "(r)"),
+            ("&lt;", "<"),
+            ("&gt;", ">"),
+            ("&amp;", "&"),
+    ):
+        bbcode = bbcode.replace(char, substitution)
+
+    return bbcode.strip(" ")
+
+
+def bbcode_to_html(bbcode: str) -> str:
+    def render_url(_tag_name, value: str, options: dict[str, str], _parent, _context) -> str:
+        return f'<a class="auto_link named_url" href="{options.get("url", "#")}">{value}</a>'
+
+    def render_color(_tag_name, value, options, _parent, _context) -> str:
+        return f'<span class=bbcode style="color:{options.get("color", "inherit")};">{value}</span>'
+
+    def render_quote(_tag_name, value: str, options: dict[str, str], _parent, _context) -> str:
+        author: str = options.get("quote", "")
+        author = f"<span class=bbcode_quote_name>{author} wrote:</span>" if author else ""
+        return f'<span class="bbcode bbcode_quote">{author}{value}</span>'
+
+    def render_tags(tag_name: str, value: str, options: dict[str, str], _parent, _context) -> str:
+        if not options and tag_name.islower():
+            return f"<{tag_name}>{value}</{tag_name}>"
+        return f"[{tag_name} {' '.join(f'{k}={v}' if v else k for k, v in options.items())}]{value}"
+
+    def render_tag(_tag_name, value: str, options: dict[str, str], _parent, _context) -> str:
+        name, *classes = options["tag"].split(".")
+        return f'<{name} class="{" ".join(classes)}">{value}</{name}>'
+
+    def parse_extra(page: BeautifulSoup) -> BeautifulSoup:
+        child: NavigableString
+        child_new: Tag
+        has_match: bool = True
+        while has_match:
+            has_match = False
+            for child in [c for e in page.select("*:not(a)") for c in e.children if isinstance(c, NavigableString)]:
+                if m_ := match(rf"(.*):({'|'.join(smilie_icons)}):(.*)", child):
+                    has_match = True
+                    child_new = Tag(name="i", attrs={"class": f"smilie {m_[2]}"})
+                    child.replaceWith(m_[1], child_new, m_[3])
+                elif m_ := match(r"(.*)(?:@([a-zA-Z0-9.~_-]+)|:link([a-zA-Z0-9.~_-]+):)(.*)", child):
+                    has_match = True
+                    child_new = Tag(name="a", attrs={"class": "linkusername", "href": f"/user/{m_[2] or m_[3]}"})
+                    child_new.insert(0, m_[2] or m_[3])
+                    child.replaceWith(m_[1], child_new, m_[4])
+                elif m_ := match(r"(.*):(?:icon([a-zA-Z0-9.~_-]+)|([a-zA-Z0-9.~_-]+)icon):(.*)", child):
+                    has_match = True
+                    user: str = m_[2] or m_[3] or ""
+                    child_new = Tag(name="a", attrs={"class": "iconusername", "href": f"/user/{user}"})
+                    child_new_img: Tag = Tag(name="img",
+                                             attrs={"alt": user, "title": user,
+                                                    "src": f"//a.furaffinity.net/{datetime.now():%Y%m%d}/{user}.gif"})
+                    child_new.insert(0, child_new_img)
+                    if m_[2]:
+                        child_new.insert(1, f"\xA0{m_[2]}")
+                    child.replaceWith(m_[1], child_new, m_[4])
+                elif m_ := match(r"(.*)\[ *(?:(\d+)|-)?, *(?:(\d+)|-)? *, *(?:(\d+)|-)? *](.*)", child):
+                    has_match = True
+                    child_new = Tag(name="span", attrs={"class": "parsed_nav_links"})
+                    child_new_1: Union[Tag, str] = "<<<\xA0PREV"
+                    child_new_2: Union[Tag, str] = "FIRST"
+                    child_new_3: Union[Tag, str] = "NEXT\xA0>>>"
+                    if m_[2]:
+                        child_new_1 = Tag(name="a", attrs={"href": f"/view/{m_[2]}"})
+                        child_new_1.insert(0, "<<<\xA0PREV")
+                    if m_[3]:
+                        child_new_2 = Tag(name="a", attrs={"href": f"/view/{m_[3]}"})
+                        child_new_2.insert(0, "<<<\xA0FIRST")
+                    if m_[4]:
+                        child_new_3 = Tag(name="a", attrs={"href": f"/view/{m_[4]}"})
+                        child_new_3.insert(0, "NEXT\xA0>>>")
+                    child_new.insert(0, child_new_1)
+                    child_new.insert(1, "\xA0|\xA0")
+                    child_new.insert(2, child_new_2)
+                    child_new.insert(3, "\xA0|\xA0")
+                    child_new.insert(4, child_new_3)
+                    child.replaceWith(m_[1], child_new, m_[5])
+
+        for p in page.select("p"):
+            p.replaceWith(*p.children)
+
+        return page
+
+    parser: BBCodeParser = BBCodeParser(install_defaults=False, replace_links=False, replace_cosmetic=True)
+    parser.REPLACE_ESCAPE = (
+        ("&", "&amp;"),
+        ("<", "&lt;"),
+        (">", "&gt;"),
+    )
+    parser.REPLACE_COSMETIC = (
+        ("(c)", "&copy;"),
+        ("(r)", "&reg;"),
+        ("(tm)", "&trade;"),
+    )
+
+    for tag in ("i", "b", "u", "s", "sub", "sup", "h1", "h2", "h3", "h3", "h4", "h5", "h6"):
+        parser.add_formatter(tag, render_tags)
+    for align in ("left", "center", "right"):
+        parser.add_simple_formatter(align, f'<code class="bbcode bbcode_{align}">%(value)s</code>')
+
+    parser.add_simple_formatter("spoiler", '<span class="bbcode bbcode_spoiler">%(value)s</span>')
+    parser.add_simple_formatter("url", '<a class="auto_link named_link">%(value)s</a>')
+    parser.add_simple_formatter(
+        "iconusername",
+        f'<a class=iconusername href="/user/%(value)s">'
+        f'<img alt="%(value)s" title="%(value)s" src="//a.furaffinity.net/{datetime.now():%Y%m%d}/%(value)s.gif">'
+        f'%(value)s'
+        f'</a>'
+    )
+    parser.add_simple_formatter(
+        "usernameicon",
+        f'<a class=iconusername href="/user/%(value)s">'
+        f'<img alt="%(value)s" title="%(value)s" src="//a.furaffinity.net/{datetime.now():%Y%m%d}/%(value)s.gif">'
+        f'</a>'
+    )
+    parser.add_simple_formatter("linkusername", '<a class=linkusername href="/user/%(value)s">%(value)s</a>')
+    parser.add_simple_formatter("hr", "<hr>", standalone=True)
+
+    parser.add_formatter("url", render_url)
+    parser.add_formatter("color", render_color)
+    parser.add_formatter("quote", render_quote)
+    parser.add_formatter("tag", render_tag)
+
+    bbcode = sub(r"-{5,}", "[hr]", bbcode)
+
+    result_page: BeautifulSoup = parse_extra(parse_page(parser.format(bbcode)))
+    return (result_page.select_one("html > body") or result_page).decode_contents()
 
 
 def parse_mentions(tag: Tag) -> list[str]:
@@ -254,8 +395,8 @@ def parse_journal_page(journal_page: BeautifulSoup) -> dict[str, Any]:
         if match(r"^[A-Za-z]+ \d+,.*$", get_attr(tag_date, "title"))
         else tag_date.text.strip()
     )
-    header: str = inner_html(tag_header) if tag_header else ""
-    footer: str = inner_html(tag_footer) if tag_footer else ""
+    header: str = clean_html(inner_html(tag_header)) if tag_header else ""
+    footer: str = clean_html(inner_html(tag_footer)) if tag_footer else ""
     content: str = clean_html(inner_html(tag_content))
     mentions: list[str] = parse_mentions(tag_content)
     comments: int = int(tag_comments.text.strip())
@@ -379,6 +520,8 @@ def parse_submission_page(sub_page: BeautifulSoup) -> dict[str, Any]:
     assert tag_prev is not None, _raise_exception(ParsingError("Missing prev tag"))
     assert tag_next is not None, _raise_exception(ParsingError("Missing next tag"))
 
+    tag_footer: Optional[Tag] = tag_description.select_one("div.submission-footer")
+
     id_: int = int(get_attr(tag_id, "content").strip("/").split("/")[-1])
     title: str = tag_title.text.strip()
     date: datetime = parse_date(
@@ -395,6 +538,12 @@ def parse_submission_page(sub_page: BeautifulSoup) -> dict[str, Any]:
     comment_count: int = int(tag_comment_count.text.strip())
     favorites: int = int(tag_favorites.text.strip())
     type_: str = tag_type["class"][0][18:]
+    footer: str = ""
+    if tag_footer:
+        if tag_footer_hr := tag_footer.select_one("hr"):
+            tag_footer_hr.decompose()
+        footer = clean_html(inner_html(tag_footer))
+        tag_footer.decompose()
     description: str = clean_html(inner_html(tag_description))
     mentions: list[str] = parse_mentions(tag_description)
     folder: str = m.group(1).lower() if (m := match(r"^/(scraps|gallery)/.*$", get_attr(tag_folder, "href"))) else ""
@@ -431,6 +580,7 @@ def parse_submission_page(sub_page: BeautifulSoup) -> dict[str, Any]:
         "comment_count": comment_count,
         "favorites": favorites,
         "type": type_,
+        "footer": footer,
         "description": description,
         "mentions": mentions,
         "folder": folder,
@@ -645,10 +795,11 @@ def parse_user_journals(journals_page: BeautifulSoup) -> dict[str, Any]:
     }
 
 
-def parse_watchlist(watch_page: BeautifulSoup) -> tuple[list[tuple[str, str]], bool]:
+def parse_watchlist(watch_page: BeautifulSoup) -> tuple[list[tuple[str, str]], int]:
     tags_users: list[Tag] = watch_page.select("div.watch-list-items")
     tag_next: Optional[Tag] = watch_page.select_one("section div.floatright form[method=get]")
+    match_next: Optional[Match] = watchlist_next_regexp.match(get_attr(tag_next, "action")) if tag_next else None
     return (
         [((u := t.text.strip().replace(" ", ""))[0], u[1:]) for t in tags_users],
-        watchlist_next_regexp.match(get_attr(tag_next, "action")) is not None if tag_next else False
+        int(match_next[1]) if match_next else 0,
     )
