@@ -6,12 +6,12 @@ from .connection import join_url
 from .connection import root
 from .exceptions import _raise_exception
 from .parse import BeautifulSoup
+from .parse import Tag
 from .parse import check_page_raise
 from .parse import html_to_bbcode
 from .parse import parse_comments
 from .parse import parse_submission_figure
 from .parse import parse_submission_page
-from .parse import Tag
 from .user import UserPartial
 
 
@@ -43,6 +43,9 @@ class SubmissionBase:
         self.title: str = ""
         self.author: UserPartial = UserPartial()
 
+    def __hash__(self) -> int:
+        return hash(self.id)
+
     def __eq__(self, other) -> bool:
         if isinstance(other, SubmissionBase):
             return other.id == self.id
@@ -175,6 +178,7 @@ class Submission(SubmissionBase):
         self.stats: SubmissionStats = SubmissionStats(0, 0, 0)
         self.type: str = ""
         self.description: str = ""
+        self.footer: str = ""
         self.mentions: list[str] = []
         self.folder: str = ""
         self.user_folders: list[SubmissionUserFolder] = []
@@ -202,6 +206,7 @@ class Submission(SubmissionBase):
         yield "stats", self.stats._asdict()
         yield "type", self.type
         yield "description", self.description
+        yield "footer", self.footer
         yield "mentions", self.mentions
         yield "folder", self.folder
         yield "user_folders", [f._asdict() for f in self.user_folders]
@@ -216,7 +221,21 @@ class Submission(SubmissionBase):
 
     @property
     def description_bbcode(self) -> str:
-        return html_to_bbcode(self.description, special_characters=True)
+        """
+        The submission description formatted to BBCode
+
+        :return: BBCode description
+        """
+        return html_to_bbcode(self.description)
+
+    @property
+    def footer_bbcode(self) -> str:
+        """
+        The submission footer formatted to BBCode
+
+        :return: BBCode footer
+        """
+        return html_to_bbcode(self.footer)
 
     def parse(self, submission_page: BeautifulSoup = None):
         """
@@ -249,6 +268,7 @@ class Submission(SubmissionBase):
         self.stats = SubmissionStats(parsed["views"], parsed["comment_count"], parsed["favorites"])
         self.type = parsed["type"]
         self.description = parsed["description"]
+        self.footer = parsed["footer"]
         self.mentions = parsed["mentions"]
         self.folder = parsed["folder"]
         self.user_folders = [SubmissionUserFolder(*f) for f in parsed["user_folders"]]
